@@ -111,9 +111,12 @@ import org.apache.hadoop.hdfs.protocol.HdfsFileStatus;
 import org.apache.hadoop.hdfs.server.blockmanagement.BlockStoragePolicySuite;
 import org.apache.hadoop.io.NullWritable;
 import org.apache.hadoop.mapreduce.Job;
+import org.apache.hadoop.mapreduce.JobContext;
 import org.apache.hadoop.mapreduce.Mapper;
+import org.apache.hadoop.mapreduce.OutputCommitter;
 import org.apache.hadoop.mapreduce.RecordWriter;
 import org.apache.hadoop.mapreduce.TaskAttemptContext;
+import org.apache.hadoop.mapreduce.lib.output.FileOutputCommitter;
 import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
 import org.junit.ClassRule;
 import org.junit.Ignore;
@@ -810,7 +813,7 @@ public class TestHFileOutputFormat2 {
 
   /**
    * Test for {@link HFileOutputFormat2#createFamilyCompressionMap(Configuration)}. Tests that the
-   * family compression map is correctly serialized into and deserialized from configuration n
+   * family compression map is correctly serialized into and deserialized from configuration
    */
   @Ignore("Goes zombie too frequently; needs work. See HBASE-14563")
   @Test
@@ -872,7 +875,7 @@ public class TestHFileOutputFormat2 {
 
   /**
    * Test for {@link HFileOutputFormat2#createFamilyBloomTypeMap(Configuration)}. Tests that the
-   * family bloom type map is correctly serialized into and deserialized from configuration n
+   * family bloom type map is correctly serialized into and deserialized from configuration
    */
   @Ignore("Goes zombie too frequently; needs work. See HBASE-14563")
   @Test
@@ -931,7 +934,7 @@ public class TestHFileOutputFormat2 {
 
   /**
    * Test for {@link HFileOutputFormat2#createFamilyBlockSizeMap(Configuration)}. Tests that the
-   * family block size map is correctly serialized into and deserialized from configuration n
+   * family block size map is correctly serialized into and deserialized from configuration
    */
   @Ignore("Goes zombie too frequently; needs work. See HBASE-14563")
   @Test
@@ -994,7 +997,7 @@ public class TestHFileOutputFormat2 {
   /**
    * Test for {@link HFileOutputFormat2#createFamilyDataBlockEncodingMap(Configuration)}. Tests that
    * the family data block encoding map is correctly serialized into and deserialized from
-   * configuration n
+   * configuration
    */
   @Ignore("Goes zombie too frequently; needs work. See HBASE-14563")
   @Test
@@ -1583,6 +1586,59 @@ public class TestHFileOutputFormat2 {
     }
   }
 
+  @Test
+  public void itGetsWorkPathHadoop2() throws Exception {
+    Configuration conf = new Configuration(this.util.getConfiguration());
+    Job job = new Job(conf);
+    FileOutputCommitter committer =
+      new FileOutputCommitter(new Path("/test"), createTestTaskAttemptContext(job));
+    assertEquals(committer.getWorkPath(), HFileOutputFormat2.getWorkPath(committer));
+  }
+
+  @Test
+  public void itGetsWorkPathHadoo3() {
+    Hadoop3TestOutputCommitter committer = new Hadoop3TestOutputCommitter(new Path("/test"));
+    assertEquals(committer.getWorkPath(), HFileOutputFormat2.getWorkPath(committer));
+  }
+
+  static class Hadoop3TestOutputCommitter extends OutputCommitter {
+
+    Path path;
+
+    Hadoop3TestOutputCommitter(Path path) {
+      this.path = path;
+    }
+
+    public Path getWorkPath() {
+      return path;
+    }
+
+    @Override
+    public void setupJob(JobContext jobContext) throws IOException {
+
+    }
+
+    @Override
+    public void setupTask(TaskAttemptContext taskAttemptContext) throws IOException {
+
+    }
+
+    @Override
+    public boolean needsTaskCommit(TaskAttemptContext taskAttemptContext) throws IOException {
+      return false;
+    }
+
+    @Override
+    public void commitTask(TaskAttemptContext taskAttemptContext) throws IOException {
+
+    }
+
+    @Override
+    public void abortTask(TaskAttemptContext taskAttemptContext) throws IOException {
+
+    }
+  }
+
   private static class ConfigurationCaptorConnection implements Connection {
     private static final String UUID_KEY = "ConfigurationCaptorConnection.uuid";
 
