@@ -19,13 +19,17 @@ package org.apache.hadoop.hbase.rest.model;
 
 import com.fasterxml.jackson.annotation.JsonInclude;
 import com.google.protobuf.ByteString;
+import com.google.protobuf.CodedInputStream;
+import com.google.protobuf.Message;
 import java.io.IOException;
 import java.io.Serializable;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Base64;
 import java.util.List;
 import java.util.Map;
 import java.util.NavigableSet;
+import java.util.Objects;
 import javax.xml.bind.annotation.XmlAttribute;
 import javax.xml.bind.annotation.XmlElement;
 import javax.xml.bind.annotation.XmlRootElement;
@@ -65,8 +69,8 @@ import org.apache.hadoop.hbase.filter.SubstringComparator;
 import org.apache.hadoop.hbase.filter.TimestampsFilter;
 import org.apache.hadoop.hbase.filter.ValueFilter;
 import org.apache.hadoop.hbase.filter.WhileMatchFilter;
-import org.apache.hadoop.hbase.protobuf.ProtobufUtil;
 import org.apache.hadoop.hbase.rest.ProtobufMessageHandler;
+import org.apache.hadoop.hbase.rest.RestUtil;
 import org.apache.hadoop.hbase.rest.protobuf.generated.ScannerMessage.Scanner;
 import org.apache.hadoop.hbase.security.visibility.Authorizations;
 import org.apache.hadoop.hbase.util.ByteStringer;
@@ -200,6 +204,79 @@ public class ScannerModel implements ProtobufMessageHandler, Serializable {
 
     }
 
+    /**
+     * This DTO omits the pseudo-getters in MultiRowRangeFilter.RowRange which break Jackson
+     * deserialization. It also avoids adding those as dummy JSON elements.
+     */
+    static class RowRangeModel {
+
+      protected byte[] startRow;
+
+      protected boolean startRowInclusive = true;
+
+      protected byte[] stopRow;
+
+      protected boolean stopRowInclusive = false;
+
+      public RowRangeModel() {
+      }
+
+      public RowRangeModel(MultiRowRangeFilter.RowRange rr) {
+        this.startRow = rr.getStartRow();
+        this.startRowInclusive = rr.isStartRowInclusive();
+        this.stopRow = rr.getStopRow();
+        this.stopRowInclusive = rr.isStopRowInclusive();
+      }
+
+      public MultiRowRangeFilter.RowRange build() {
+        return new MultiRowRangeFilter.RowRange(startRow, startRowInclusive, stopRow,
+          stopRowInclusive);
+      }
+
+      public byte[] getStartRow() {
+        return startRow;
+      }
+
+      public byte[] getStopRow() {
+        return stopRow;
+      }
+
+      /** Returns if start row is inclusive. */
+      public boolean isStartRowInclusive() {
+        return startRowInclusive;
+      }
+
+      /** Returns if stop row is inclusive. */
+      public boolean isStopRowInclusive() {
+        return stopRowInclusive;
+      }
+
+      @Override
+      public int hashCode() {
+        final int prime = 31;
+        int result = 1;
+        result = prime * result + Arrays.hashCode(startRow);
+        result = prime * result + Arrays.hashCode(stopRow);
+        result = prime * result + Objects.hash(startRowInclusive, stopRowInclusive);
+        return result;
+      }
+
+      @Override
+      public boolean equals(Object obj) {
+        if (this == obj) {
+          return true;
+        }
+        if (!(obj instanceof RowRangeModel)) {
+          return false;
+        }
+        RowRangeModel other = (RowRangeModel) obj;
+        return Arrays.equals(startRow, other.startRow)
+          && startRowInclusive == other.startRowInclusive && Arrays.equals(stopRow, other.stopRow)
+          && stopRowInclusive == other.stopRowInclusive;
+      }
+
+    }
+
     // A grab bag of fields, would have been a union if this were C.
     // These are null by default and will only be serialized if set (non null).
     @XmlAttribute
@@ -239,7 +316,7 @@ public class ScannerModel implements ProtobufMessageHandler, Serializable {
     @XmlElement
     public List<String> prefixes;
     @XmlElement
-    private List<RowRange> ranges;
+    private List<RowRangeModel> ranges;
     @XmlElement
     public List<Long> timestamps;
 
@@ -330,8 +407,7 @@ public class ScannerModel implements ProtobufMessageHandler, Serializable {
         case MultiRowRangeFilter:
           this.ranges = new ArrayList<>();
           for (RowRange range : ((MultiRowRangeFilter) filter).getRowRanges()) {
-            this.ranges.add(new RowRange(range.getStartRow(), range.isStartRowInclusive(),
-              range.getStopRow(), range.isStopRowInclusive()));
+            this.ranges.add(new RowRangeModel(range));
           }
           break;
         case PageFilter:
@@ -435,7 +511,11 @@ public class ScannerModel implements ProtobufMessageHandler, Serializable {
         }
           break;
         case MultiRowRangeFilter: {
-          filter = new MultiRowRangeFilter(ranges);
+          ArrayList<MultiRowRangeFilter.RowRange> rowRanges = new ArrayList<>(ranges.size());
+          for (RowRangeModel rangeModel : ranges) {
+            rowRanges.add(rangeModel.build());
+          }
+          filter = new MultiRowRangeFilter(rowRanges);
         }
           break;
         case PageFilter:
@@ -505,7 +585,7 @@ public class ScannerModel implements ProtobufMessageHandler, Serializable {
 
   /**
    * @param s the JSON representation of the filter
-   * @return the filter n
+   * @return the filter
    */
   public static Filter buildFilter(String s) throws Exception {
     FilterModel model =
@@ -516,7 +596,7 @@ public class ScannerModel implements ProtobufMessageHandler, Serializable {
 
   /**
    * @param filter the filter
-   * @return the JSON representation of the filter n
+   * @return the JSON representation of the filter
    */
   public static String stringifyFilter(final Filter filter) throws Exception {
     return getJasonProvider().locateMapper(FilterModel.class, MediaType.APPLICATION_JSON_TYPE)
@@ -526,7 +606,7 @@ public class ScannerModel implements ProtobufMessageHandler, Serializable {
   private static final byte[] COLUMN_DIVIDER = Bytes.toBytes(":");
 
   /**
-   * @param scan the scan specification n
+   * @param scan the scan specification
    */
   public static ScannerModel fromScan(Scan scan) throws Exception {
     ScannerModel model = new ScannerModel();
@@ -791,7 +871,7 @@ public class ScannerModel implements ProtobufMessageHandler, Serializable {
   }
 
   @Override
-  public byte[] createProtobufOutput() {
+  public Message messageFromObject() {
     Scanner.Builder builder = Scanner.newBuilder();
     if (!Bytes.equals(startRow, HConstants.EMPTY_START_ROW)) {
       builder.setStartRow(ByteStringer.wrap(startRow));
@@ -821,13 +901,13 @@ public class ScannerModel implements ProtobufMessageHandler, Serializable {
         builder.addLabels(label);
     }
     builder.setCacheBlocks(cacheBlocks);
-    return builder.build().toByteArray();
+    return builder.build();
   }
 
   @Override
-  public ProtobufMessageHandler getObjectFromMessage(byte[] message) throws IOException {
+  public ProtobufMessageHandler getObjectFromMessage(CodedInputStream cis) throws IOException {
     Scanner.Builder builder = Scanner.newBuilder();
-    ProtobufUtil.mergeFrom(builder, message);
+    RestUtil.mergeFrom(builder, cis);
     if (builder.hasStartRow()) {
       startRow = builder.getStartRow().toByteArray();
     }
