@@ -18,6 +18,9 @@
 package org.apache.hadoop.hbase.ipc;
 
 import com.google.errorprone.annotations.RestrictedApi;
+import io.opentelemetry.api.trace.Span;
+import io.opentelemetry.api.trace.StatusCode;
+import io.opentelemetry.context.Scope;
 import java.io.IOException;
 import java.net.InetAddress;
 import java.nio.ByteBuffer;
@@ -27,11 +30,13 @@ import java.util.Optional;
 import java.util.concurrent.atomic.AtomicInteger;
 import org.apache.hadoop.hbase.CellScanner;
 import org.apache.hadoop.hbase.DoNotRetryIOException;
+import org.apache.hadoop.hbase.HBaseServerException;
 import org.apache.hadoop.hbase.exceptions.RegionMovedException;
 import org.apache.hadoop.hbase.io.ByteBuffAllocator;
 import org.apache.hadoop.hbase.io.ByteBufferListOutputStream;
 import org.apache.hadoop.hbase.ipc.RpcServer.CallCleanup;
 import org.apache.hadoop.hbase.security.User;
+import org.apache.hadoop.hbase.trace.TraceUtil;
 import org.apache.hadoop.hbase.util.ByteBufferUtils;
 import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
@@ -103,6 +108,8 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
   // from WAL side on release
   private final AtomicInteger reference = new AtomicInteger(0x80000000);
 
+  private final Span span;
+
   @edu.umd.cs.findbugs.annotations.SuppressWarnings(value = "NP_NULL_ON_SOME_PATH",
       justification = "Can't figure why this complaint is happening... see below")
   ServerCall(int id, BlockingService service, MethodDescriptor md, RequestHeader header,
@@ -133,6 +140,7 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
     this.bbAllocator = byteBuffAllocator;
     this.cellBlockBuilder = cellBlockBuilder;
     this.reqCleanup = reqCleanup;
+    this.span = Span.current();
   }
 
   /**
@@ -150,6 +158,7 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
     // If the call was run successfuly, we might have already returned the BB
     // back to pool. No worries..Then inputCellBlock will be null
     cleanup();
+    span.end();
   }
 
   @Override
@@ -215,7 +224,7 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
     return "callId: " + this.id + " service: " + serviceName + " methodName: "
       + ((this.md != null) ? this.md.getName() : "n/a") + " size: "
       + StringUtils.TraditionalBinaryPrefix.long2String(this.size, "", 1) + " connection: "
-      + connection.toString() + " deadline: " + deadline;
+      + connection + " deadline: " + deadline;
   }
 
   @Override
@@ -226,6 +235,9 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
     }
     if (t != null) {
       this.isError = true;
+      TraceUtil.setError(span, t);
+    } else {
+      span.setStatus(StatusCode.OK);
     }
     BufferChain bc = null;
     try {
@@ -287,11 +299,12 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
     // Once a response message is created and set to this.response, this Call can be treated as
     // done. The Responder thread will do the n/w write of this message back to client.
     if (this.rpcCallback != null) {
-      try {
+      try (Scope ignored = span.makeCurrent()) {
         this.rpcCallback.run();
       } catch (Exception e) {
         // Don't allow any exception here to kill this handler thread.
         RpcServer.LOG.warn("Exception while running the Rpc Callback.", e);
+        TraceUtil.setError(span, e);
       }
     }
   }
@@ -309,6 +322,9 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
       RegionMovedException rme = (RegionMovedException) t;
       exceptionBuilder.setHostname(rme.getHostname());
       exceptionBuilder.setPort(rme.getPort());
+    } else if (t instanceof HBaseServerException) {
+      HBaseServerException hse = (HBaseServerException) t;
+      exceptionBuilder.setServerOverloaded(hse.isServerOverloaded());
     }
     // Set the exception as the result of the method invocation.
     headerBuilder.setException(exceptionBuilder.build());
