@@ -292,6 +292,7 @@ public class MergeTableRegionsProcedure
           postRollBackMergeRegions(env);
           break;
         case MERGE_TABLE_REGIONS_PREPARE:
+          rollbackPrepareMerge(env);
           break;
         default:
           throw new UnsupportedOperationException(this + " unhandled state=" + state);
@@ -514,6 +515,19 @@ public class MergeTableRegionsProcedure
     return response.hasMergeable() && response.getMergeable();
   }
 
+  /**
+   * Action for rollback a merge table after prepare merge
+   */
+  private void rollbackPrepareMerge(final MasterProcedureEnv env) throws IOException {
+    for (RegionInfo rinfo : regionsToMerge) {
+      RegionStateNode regionStateNode =
+        env.getAssignmentManager().getRegionStates().getRegionStateNode(rinfo);
+      if (regionStateNode.getState() == State.MERGING) {
+        regionStateNode.setState(State.OPEN);
+      }
+    }
+  }
+
   /**
    * Pre merge region action
    * @param env MasterProcedureEnv
@@ -639,8 +653,20 @@ public class MergeTableRegionsProcedure
    * Rollback close regions
    **/
   private void rollbackCloseRegionsForMerge(MasterProcedureEnv env) throws IOException {
-    AssignmentManagerUtil.reopenRegionsForRollback(env, Arrays.asList(regionsToMerge),
-      getRegionReplication(env), getServerName(env));
+    // At this point we should check if region was actually closed. If it was not closed then we
+    // don't need to repoen the region and we can just change the regionNode state to OPEN.
+    // if it is alredy closed then we need to do a reopen of region
+    List<RegionInfo> toAssign = new ArrayList<>();
+    for (RegionInfo rinfo : regionsToMerge) {
+      RegionStateNode regionStateNode =
+        env.getAssignmentManager().getRegionStates().getRegionStateNode(rinfo);
+      if (regionStateNode.getState() != State.MERGING) {
+        // same as before HBASE-28405
+        toAssign.add(rinfo);
+      }
+    }
+    AssignmentManagerUtil.reopenRegionsForRollback(env, toAssign, getRegionReplication(env),
+      getServerName(env));
   }
 
   private TransitRegionStateProcedure[] createUnassignProcedures(MasterProcedureEnv env)
