@@ -220,12 +220,18 @@ public class StoreScanner extends NonReversedNonLazyKeyValueScanner
     this.currentScanners.addAll(scanners);
   }
 
+  private static boolean isOnlyLatestVersionScan(Scan scan) {
+    // No need to check for Scan#getMaxVersions because live version files generated by store file
+    // writer retains max versions specified in ColumnFamilyDescriptor for the given CF
+    return !scan.isRaw() && scan.getTimeRange().getMax() == HConstants.LATEST_TIMESTAMP;
+  }
+
   /**
    * Opens a scanner across memstore, snapshot, and all StoreFiles. Assumes we are not in a
    * compaction.
    * @param store   who we scan
    * @param scan    the spec
-   * @param columns which columns we are scanning n
+   * @param columns which columns we are scanning
    */
   public StoreScanner(HStore store, ScanInfo scanInfo, Scan scan, NavigableSet<byte[]> columns,
     long readPt) throws IOException {
@@ -244,7 +250,8 @@ public class StoreScanner extends NonReversedNonLazyKeyValueScanner
       // Pass columns to try to filter out unnecessary StoreFiles.
       scanners = selectScannersFrom(store,
         store.getScanners(cacheBlocks, scanUsePread, false, matcher, scan.getStartRow(),
-          scan.includeStartRow(), scan.getStopRow(), scan.includeStopRow(), this.readPt));
+          scan.includeStartRow(), scan.getStopRow(), scan.includeStopRow(), this.readPt,
+          isOnlyLatestVersionScan(scan)));
 
       // Seek all scanners to the start of the Row (or if the exact matching row
       // key does not exist, then to the start of the next matching Row).
@@ -384,8 +391,9 @@ public class StoreScanner extends NonReversedNonLazyKeyValueScanner
   }
 
   /**
-   * Seek the specified scanners with the given key nn * @param isLazy true if using lazy seek
-   * @param isParallelSeek true if using parallel seek n
+   * Seek the specified scanners with the given key
+   * @param isLazy         true if using lazy seek
+   * @param isParallelSeek true if using parallel seek
    */
   protected void seekScanners(List<? extends KeyValueScanner> scanners, Cell seekKey,
     boolean isLazy, boolean isParallelSeek) throws IOException {
@@ -530,8 +538,8 @@ public class StoreScanner extends NonReversedNonLazyKeyValueScanner
   }
 
   /**
-   * Get the next row of values from this Store. nn * @return true if there are more rows, false if
-   * scanner is done
+   * Get the next row of values from this Store.
+   * @return true if there are more rows, false if scanner is done
    */
   @Override
   public boolean next(List<Cell> outResult, ScannerContext scannerContext) throws IOException {
@@ -912,7 +920,7 @@ public class StoreScanner extends NonReversedNonLazyKeyValueScanner
     // We need this check because it may happen that the new scanner that we get
     // during heap.next() is requiring reseek due of fake KV previously generated for
     // ROWCOL bloom filter optimization. See HBASE-19863 for more details
-    if (useRowColBloom && nextCell != null && matcher.compareKeyForNextColumn(nextCell, cell) < 0) {
+    if (useRowColBloom && nextCell != null && cell.getTimestamp() == HConstants.OLDEST_TIMESTAMP) {
       return false;
     }
     return true;
@@ -970,8 +978,9 @@ public class StoreScanner extends NonReversedNonLazyKeyValueScanner
       // Eagerly creating scanners so that we have the ref counting ticking on the newly created
       // store files. In case of stream scanners this eager creation does not induce performance
       // penalty because in scans (that uses stream scanners) the next() call is bound to happen.
-      List<KeyValueScanner> scanners = store.getScanners(sfs, cacheBlocks, get, usePread,
-        isCompaction, matcher, scan.getStartRow(), scan.getStopRow(), this.readPt, false);
+      List<KeyValueScanner> scanners =
+        store.getScanners(sfs, cacheBlocks, get, usePread, isCompaction, matcher,
+          scan.getStartRow(), scan.getStopRow(), this.readPt, false, isOnlyLatestVersionScan(scan));
       flushedstoreFileScanners.addAll(scanners);
       if (!CollectionUtils.isEmpty(memStoreScanners)) {
         clearAndClose(memStoreScannersAfterFlush);
@@ -1050,7 +1059,7 @@ public class StoreScanner extends NonReversedNonLazyKeyValueScanner
   }
 
   /**
-   * Check whether scan as expected order nnnn
+   * Check whether scan as expected order
    */
   protected void checkScanOrder(Cell prevKV, Cell kv, CellComparator comparator)
     throws IOException {
@@ -1064,8 +1073,8 @@ public class StoreScanner extends NonReversedNonLazyKeyValueScanner
   }
 
   /**
-   * Do a reseek in a normal StoreScanner(scan forward) n * @return true if scanner has values left,
-   * false if end of scanner n
+   * Do a reseek in a normal StoreScanner(scan forward)
+   * @return true if scanner has values left, false if end of scanner
    */
   protected boolean seekAsDirection(Cell kv) throws IOException {
     return reseek(kv);
@@ -1156,7 +1165,7 @@ public class StoreScanner extends NonReversedNonLazyKeyValueScanner
   /**
    * Seek storefiles in parallel to optimize IO latency as much as possible
    * @param scanners the list {@link KeyValueScanner}s to be read from
-   * @param kv       the KeyValue on which the operation is being requested n
+   * @param kv       the KeyValue on which the operation is being requested
    */
   private void parallelSeek(final List<? extends KeyValueScanner> scanners, final Cell kv)
     throws IOException {
