@@ -20,6 +20,8 @@ package org.apache.hadoop.hbase.master.balancer;
 import static junit.framework.TestCase.assertNotNull;
 import static junit.framework.TestCase.assertTrue;
 import static org.junit.Assert.assertNull;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.when;
 
 import java.io.IOException;
 import java.util.Arrays;
@@ -38,6 +40,7 @@ import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.hbase.ServerName;
 import org.apache.hadoop.hbase.client.RegionInfo;
 import org.apache.hadoop.hbase.client.RegionReplicaUtil;
+import org.apache.hadoop.hbase.master.MasterServices;
 import org.apache.hadoop.hbase.master.RackManager;
 import org.apache.hadoop.hbase.master.RegionPlan;
 import org.apache.hadoop.hbase.testclassification.MasterTests;
@@ -77,9 +80,12 @@ public class TestStochasticLoadBalancerHeterogeneousCost extends BalancerTestBas
     BalancerTestBase.conf.set(
       HeterogeneousRegionCountCostFunction.HBASE_MASTER_BALANCER_HETEROGENEOUS_RULES_FILE,
       RULES_FILE);
-    BalancerTestBase.loadBalancer = new StochasticLoadTestBalancer();
-    BalancerTestBase.loadBalancer.setConf(BalancerTestBase.conf);
-    BalancerTestBase.loadBalancer.getCandidateGenerators().add(new FairRandomCandidateGenerator());
+    loadBalancer = new StochasticLoadTestBalancer();
+    MasterServices services = mock(MasterServices.class);
+    when(services.getConfiguration()).thenReturn(conf);
+    BalancerTestBase.loadBalancer.setMasterServices(services);
+    loadBalancer.initialize();
+    loadBalancer.getCandidateGenerators().add(new FairRandomCandidateGenerator());
   }
 
   @Test
@@ -200,9 +206,8 @@ public class TestStochasticLoadBalancerHeterogeneousCost extends BalancerTestBas
         final HeterogeneousRegionCountCostFunction cf =
           new HeterogeneousRegionCountCostFunction(conf);
         assertNotNull(cf);
-        BaseLoadBalancer.Cluster cluster =
-          new BaseLoadBalancer.Cluster(serverMap, null, null, null);
-        cf.init(cluster);
+        BalancerClusterState cluster = new BalancerClusterState(serverMap, null, null, null);
+        cf.prepare(cluster);
 
         // checking that we all hosts have a number of regions below their limit
         for (final ServerAndLoad serverAndLoad : balancedCluster) {
@@ -283,14 +288,13 @@ public class TestStochasticLoadBalancerHeterogeneousCost extends BalancerTestBas
     return new ServerAndLoad(sn, 0);
   }
 
-  static class FairRandomCandidateGenerator
-    extends StochasticLoadBalancer.RandomCandidateGenerator {
+  static class FairRandomCandidateGenerator extends RandomCandidateGenerator {
 
     @Override
-    public BaseLoadBalancer.Cluster.Action pickRandomRegions(BaseLoadBalancer.Cluster cluster,
-      int thisServer, int otherServer) {
+    public BalanceAction pickRandomRegions(BalancerClusterState cluster, int thisServer,
+      int otherServer) {
       if (thisServer < 0 || otherServer < 0) {
-        return BaseLoadBalancer.Cluster.NullAction;
+        return BalanceAction.NULL_ACTION;
       }
 
       int thisRegion = pickRandomRegion(cluster, thisServer, 0.5);
@@ -300,7 +304,7 @@ public class TestStochasticLoadBalancerHeterogeneousCost extends BalancerTestBas
     }
 
     @Override
-    BaseLoadBalancer.Cluster.Action generate(BaseLoadBalancer.Cluster cluster) {
+    BalanceAction generate(BalancerClusterState cluster) {
       return super.generate(cluster);
     }
   }
